// **************************************************************************************
// el Comandante                                                    guerilla.bplaced.net
// --------------------------------------------------------------------------------------
// Information:
// VSPI CD-Card Breakout Board, Custom VSPI Pins
// Load and display image file (565 format) from SD-Card
// The desired (included) images must be copied to the specified SD-Card directory
// Because the 565 format there is no width or height information available
// In this example the width and the height will be extracted from the filename
// The format of the filename must be: ?xxx_yyy.raw
// ? = free char, xxx = width (3 chars), underscore, yyy = height (3 chars), .raw = extension
//
// UTFT ImageConverter565(Windows): http://www.rinkydinkelectronics.com/library.php?id=51
// Don't use the UTFT online converter, it isn't working
// --------------------------------------------------------------------------------------
// Used Libraries:
// TFT_eSPI --> https://github.com/Bodmer/TFT_eSPI
// **************************************************************************************

#include <SPI.h>
#include <FS.h>
#include <SD.h>
#include <TFT_eSPI.h>                                     // TFT_espi library
TFT_eSPI tft          = TFT_eSPI();

// ######################################################################################
// Change values in this area to match your envoirement and desires
// --------------------------------------------------------------------------------------
// Custom VSPI Pins for the SD-Card breakout board - must match the hardware
#define SCK   27
#define MISO  32
#define MOSI  33
#define CS    2
// Source directory on the SD-Card for the image files
String    sdcardDirectory = "/Temp2600";
// Delay in ms for messages
uint16_t  messageDelay = 2000;
// ######################################################################################

// Vars
uint32_t  w;
uint32_t  h;
uint32_t  z;
uint32_t  vspiBusFrequency  = 2000000;                    // >= 4000000 sd-card mount could fail
File      file;                                           // file object
String    szFilepath;               	                    // image file to load
uint32_t  imgWidth[11]      = {800, 640, 480, 427, 320, 240, 180, 160, 128, 85, 43};
uint32_t  imgHeight[11]     = {600, 480, 360, 320, 240, 180, 135, 120, 96, 64, 32};
char      fileList[11][14]  = {"/V800_600.raw", "/V640_480.raw", "/V480_360.raw", "/V427_320.raw", "/V320_240.raw", "/V240_180.raw", "/V180_135.raw", "/V160_120.raw", "/V128_096.raw", "/V085_064.raw", "/V043_032.raw"};
uint8_t*  pBuffer8;                                       // byte buffer, read image
uint16_t* pBuffer16;                                      // word buffer, push image


// Create a new SPI class on HSPI or VSPI 
SPIClass vspi = SPIClass(VSPI);


// Setup
void setup()
{
  pinMode(CS, OUTPUT);;                                   // set up slave select pin CS as output
  
  // Init the SPI communication protocol and the SD-Card before TFT_eSPI, so the chip select is defined
  z = 0;
  vspi.begin(SCK, MISO, MOSI, CS);
  if (!SD.begin(CS, vspi, vspiBusFrequency))
  {z = 1;}

  tft.init();                                             // TFT_eSPI
  digitalWrite(TFT_BL, 0);                                // switch backlight off
  tft.setRotation(3);                                     // set the desired rotation
  tft.fillScreen(0x0000);
  tft.setTextColor(0xFFFE, 0x0000);
  tft.setTextDatum(TL_DATUM);
  tft.setTextFont(2);
  tft.setTextSize(1);
  tft.setCursor(0, 16);
  digitalWrite(TFT_BL, 1);                                // switch backlight on

  // SD-Card - check error on begin
  if(z == 1)
  {
    tft.println("Card Mount Error");
    return;
  }

  // SD-Card - get type
  if(SD.cardType() == CARD_NONE)
  {
    tft.println("No SD-Card");
    return;
  }

  // Select the image file that closest match the display width and height
  w = tft.width();  
  h = tft.height();
  for (z = 0; z < 11; z++)
  {
    if (w >= imgWidth[z] && h >= imgHeight[z])
    {
      szFilepath = sdcardDirectory + fileList[z];
      break;
    }
  }

  // Extract image width and height from filepath
  tft.println(szFilepath);
  z = szFilepath.lastIndexOf('_');
  if (z != - 1 && z >= 3 && sizeof(szFilepath) >= 7)
  {
    w = szFilepath.substring(z - 3, z).toInt();
    h = szFilepath.substring(z + 1, z + 4).toInt();
    tft.print("Width: ");
    tft.println(w);
    tft.print("Height: ");
    tft.println(h);
  }
  else
  {
    tft.println("Filepath Error");
    return;
  }

  // SD-Card - open file
  file = SD.open(szFilepath, FILE_READ);
  if (file)
  {
    z = file.size();
    tft.print("Filesize: ");
    tft.println(z);
    delay (messageDelay);                                 // delay to display file data
    pBuffer8 = (byte*)malloc(z);                          // allocate byte buffer, read image
    while (file.available())
    {file.read(pBuffer8, z);}
    file.close();
    pBuffer16 = (uint16_t*)malloc(z);                     // allocate word buffer, push image
    memcpy(pBuffer16, pBuffer8, z);
    //tft.setSwapBytes (true);                              // swap the byte order for pushImage() - corrects endianness
    tft.pushImage (0, 0, w, h, pBuffer16);
    free(pBuffer16);                                      // free allocated byte buffer, read image
    free(pBuffer8);                                       // free allocated word buffer, push image
  }
  else
  {
    tft.println("Open File Error");
    return;
  }
}


// Loop
void loop()
{
// notting to do 
}
